"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decrypt = exports.encrypt = exports.getDataLength = void 0;
const crypto_1 = require("crypto");
const algorithm = "aes-256-cbc";
const validateSecrets = (secrets) => {
    if (!(secrets instanceof Array) || secrets.length === 0) {
        return false;
    }
    for (const secret of secrets) {
        if (!secret.message || !secret.passphrase) {
            return false;
        }
    }
    return true;
};
/**
 * Get data length of message
 * @param message message
 * @returns data length
 */
const getDataLength = (message) => {
    const key = (0, crypto_1.randomBytes)(32);
    const iv = (0, crypto_1.randomBytes)(16);
    const cipher = (0, crypto_1.createCipheriv)(algorithm, key, iv);
    const encrypted = cipher.update(message);
    const buffer = Buffer.concat([encrypted, cipher.final()]);
    return buffer.toString("base64").length;
};
exports.getDataLength = getDataLength;
/**
 * Encrypt secrets using Blockcrypt
 * @param secrets secrets
 * @param kdf key derivation function
 * @param headersLength optional, headers length in increments of 8 (defaults to 128)
 * @param dataLength optional, data length in increments of 8 (defaults to first secret ciphertext buffer length * 2 rounded to nearest upper increment of 128)
 * @param salt optional, salt used for deterministic unit tests
 * @param iv optional, initialization vector used for deterministic unit tests
 * @returns block
 */
const encrypt = async (secrets, kdf, headersLength, dataLength, salt, iv) => {
    if (!validateSecrets(secrets)) {
        throw new Error("Invalid secrets");
    }
    if (headersLength && headersLength % 8 !== 0) {
        throw new Error("Invalid headers length");
    }
    else if (!headersLength) {
        headersLength = 128;
    }
    if (dataLength && dataLength % 8 !== 0) {
        throw new Error("Invalid data length");
    }
    if (!salt) {
        salt = (0, crypto_1.randomBytes)(16);
    }
    if (!iv) {
        iv = (0, crypto_1.randomBytes)(16);
    }
    let headersBuffers = [];
    let dataBuffers = [];
    let dataStart = 0;
    for (const [index, secret] of secrets.entries()) {
        const key = await kdf(secret.passphrase, salt.toString("base64"));
        const dataCipher = (0, crypto_1.createCipheriv)(algorithm, key, iv);
        const dataEncrypted = dataCipher.update(secret.message);
        const dataBuffer = Buffer.concat([dataEncrypted, dataCipher.final()]);
        const dataBufferLength = dataBuffer.length;
        const headersCipher = (0, crypto_1.createCipheriv)(algorithm, key, iv);
        const headersEncrypted = headersCipher.update(`${dataStart}:${dataBufferLength}`);
        const headersBuffer = Buffer.concat([
            headersEncrypted,
            headersCipher.final(),
        ]);
        headersBuffers.push(headersBuffer);
        dataBuffers.push(dataBuffer);
        dataStart += dataBufferLength;
        if (!dataLength && index === 0) {
            dataLength =
                Math.ceil((dataBuffers[0].toString("base64").length * 2) / 128) * 128;
        }
    }
    let data = Buffer.concat(dataBuffers).toString("base64");
    const unpaddedDataLength = data.length;
    if (unpaddedDataLength > dataLength) {
        throw new Error("Data too long for data length");
    }
    dataBuffers.push((0, crypto_1.randomBytes)((dataLength - unpaddedDataLength) * 0.75));
    data = Buffer.concat(dataBuffers).toString("base64");
    let headers = Buffer.concat(headersBuffers).toString("base64");
    const unpaddedHeadersLength = headers.length;
    if (unpaddedHeadersLength > headersLength) {
        throw new Error("Headers too long for headers length");
    }
    headersBuffers.push((0, crypto_1.randomBytes)((headersLength - unpaddedHeadersLength) * 0.75));
    headers = Buffer.concat(headersBuffers).toString("base64");
    return {
        salt: salt.toString("base64"),
        iv: iv.toString("base64"),
        headers: headers,
        data: data,
    };
};
exports.encrypt = encrypt;
/**
 * Decrypt secret encrypted using Blockcrypt
 * @param passphrase passphrase
 * @param salt salt
 * @param iv initialization vector
 * @param headers headers
 * @param data data
 * @param kdf key derivation function
 * @returns message
 */
const decrypt = async (passphrase, salt, iv, headers, data, kdf) => {
    const key = await kdf(passphrase, salt);
    const headersBuffer = Buffer.from(headers, "base64");
    const dataBuffer = Buffer.from(data, "base64");
    let headerStart = 0;
    let header = null;
    while (headerStart < headersBuffer.length) {
        for (let headerEnd = headersBuffer.length; headerEnd > headerStart; headerEnd--) {
            try {
                const headersDecipher = (0, crypto_1.createDecipheriv)(algorithm, key, Buffer.from(iv, "base64"));
                const headersDecrypted = headersDecipher.update(headersBuffer.subarray(headerStart, headerEnd));
                const headerBuffer = Buffer.concat([
                    headersDecrypted,
                    headersDecipher.final(),
                ]);
                const string = headerBuffer.toString();
                if (!string.match(/�/)) {
                    header = string;
                }
                if (header) {
                    break;
                }
            }
            catch (error) { }
        }
        if (header) {
            break;
        }
        headerStart++;
    }
    if (!header) {
        throw new Error("Header not found");
    }
    try {
        const dataDecipher = (0, crypto_1.createDecipheriv)(algorithm, key, Buffer.from(iv, "base64"));
        const [dataStart, dataLength] = header.split(":");
        const dataDecrypted = dataDecipher.update(dataBuffer.subarray(parseInt(dataStart), parseInt(dataStart) + parseInt(dataLength)));
        const headerBuffer = Buffer.concat([dataDecrypted, dataDecipher.final()]);
        const secret = headerBuffer.toString();
        if (!secret) {
            throw new Error("Secret not found");
        }
        return secret;
    }
    catch (error) { }
};
exports.decrypt = decrypt;
//# sourceMappingURL=index.js.map